// TODO: Create events that only run once?
// EventDirector v0.3.219

function EventDirector(hContainer, hName, tar, evtName, fun, ceptor, cat) {
  this.handlerContainer = hContainer;
  this.handlerName = hName;
  this.target = tar;
  this.eventName = evtName;
  this.funct = fun;
  this.receptor = ceptor;
  // cat format: {owner: "ownerName", groups: ["group1", "group2", "group3" ...]}
  this.categories = cat;
  this.isOn = false;

  this.markedForDeletion = false;
}

EventDirector.prototype.createEvent = function() {
  if(this.handlerContainer[this.handlerName] == undefined) {
    this.handlerContainer[this.handlerName] = this.target.on(this.eventName, this.funct);
    this.isOn = true;
  }

  return this.isOn;
}

EventDirector.prototype.turnOnEvent = function() {
  if(!this.markedForDeletion)
    if(!this.isOn) {
      this.handlerContainer[this.handlerName] = this.target.on(this.eventName, this.funct);
      this.isOn = true;
    }

  return this.isOn;
}

EventDirector.prototype.turnOffEvent = function() {
  //console.log("Trying to stop: "+this.handlerName);
  if(!this.markedForDeletion)
    if(this.isOn) {
      this.target.off(this.eventName, this.handlerContainer[this.handlerName]);
      this.isOn = false;
    }

  return this.isOn;
}

EventDirector.prototype.changeEventFunction = function(newFun) {
  // Return previous function?
  if(!this.markForDeletion) {
    this.turnOffEvent();
    delete this.handlerContainer[this.handlerName];
    this.funct = newFun;
    this.createEvent();
  }
}

EventDirector.prototype.markForDeletion = function() {
  this.turnOffEvent();
  delete this.handlerContainer[this.handlerName];

  this.markedForDeletion = true;
  delete this.handlerContainer;
  delete this.handlerName;
  delete this.target;
  delete this.eventName;
  delete this.funct;
  delete this.receptor;
  delete this.categories;
  delete this.isOn;

  return this.markedForDeletion;
}

// Receives an array of EventDirectors, and puts them in the own elements property
function EventsArray(eventDirectorsObjectList) {
  this.elements = eventDirectorsObjectList;
}

EventsArray.prototype.deleteMarked = function() {
  for(prop in this.elements) {
    if(this.elements.hasOwnProperty(prop) && (typeof this.elements[prop] != 'function'))
      if(this.elements[prop].markedForDeletion)
        delete this.elements[prop];
  }
}

EventsArray.prototype.turnOnAllEvents = function(evtName) {
  for(prop in this.elements) {
    if(this.elements.hasOwnProperty(prop) && (typeof this.elements[prop] != 'function'))
      if(!(this.elements[prop].eventName == undefined) && (this.elements[prop].eventName == evtName))
        if(!(this.elements == undefined) && !(this.elements[prop] == undefined))
          this.elements[prop].turnOnEvent();
  }
}

EventsArray.prototype.turnOffAllEvents = function(evtName) {
  for(prop in this.elements) {
    if(this.elements.hasOwnProperty(prop) && (typeof this.elements[prop] != 'function'))
      if(!(this.elements[prop].eventName == undefined) && (this.elements[prop].eventName == evtName))
        if(!(this.elements == undefined) && !(this.elements[prop] == undefined))
          this.elements[prop].turnOffEvent();
  }
}

EventsArray.prototype.turnOffAndDeleteAllEvents = function(evtName) {
  for(prop in this.elements) {
    if(this.elements.hasOwnProperty(prop) && (typeof this.elements[prop] != 'function'))
      if(!(this.elements[prop].eventName == undefined) && (this.elements[prop].eventName == evtName)) {
        if(!(this.elements == undefined) && !(this.elements[prop] == undefined)) {
          // markForDeletion turns off the event and then marks it for deletion
          this.elements[prop].markForDeletion();
          delete this.elements[prop];
        }
      }
  }
}

EventsArray.prototype.turnOnEventsOfGroup = function(grpName) {
  for(prop in this.elements) {
    if(this.elements.hasOwnProperty(prop) && (typeof this.elements[prop] != 'function'))
      if(!(this.elements[prop].categories == undefined) && !(this.elements[prop].categories.groups == undefined) && (this.elements[prop].categories.groups instanceof Array))
        var groupsLength = this.elements[prop].categories.groups.length;
        for(var i=0; i<groupsLength; i++)
          if(grpName == this.elements[prop].categories.groups[i])
            if(this.elements[prop])
              this.elements[prop].turnOnEvent();
  }
}

EventsArray.prototype.turnOffEventsOfGroup = function(grpName) {
  for(prop in this.elements) {
    if(this.elements.hasOwnProperty(prop) && (typeof this.elements[prop] != 'function'))
      if(!(this.elements[prop].categories == undefined) && !(this.elements[prop].categories.groups == undefined) && (this.elements[prop].categories.groups instanceof Array))
        var groupsLength = this.elements[prop].categories.groups.length;
        for(var i=0; i<groupsLength; i++)
          if(grpName == this.elements[prop].categories.groups[i])
            if(this.elements[prop])
              this.elements[prop].turnOffEvent();
  }
}

EventsArray.prototype.turnOffAndDeleteEventsOfGroup = function(grpName) {
  for(prop in this.elements) {
    if(this.elements.hasOwnProperty(prop) && (typeof this.elements[prop] != 'function'))
      if(!(this.elements[prop].categories == undefined) && !(this.elements[prop].categories.groups == undefined) && (this.elements[prop].categories.groups instanceof Array))
        var groupsLength = this.elements[prop].categories.groups.length;
        for(var i=0; i<groupsLength; i++)
          if(!(this.elements[prop] == undefined) && !(this.elements[prop].categories == undefined) && !(this.elements[prop].categories.groups == undefined))
            if(grpName == this.elements[prop].categories.groups[i]) {
              if(this.elements[prop]) {
                this.elements[prop].markForDeletion();
                delete this.elements[prop];
              }
            }
  }
}

EventsArray.prototype.turnOnEventsOfOwner = function(ownName) {
  for(prop in this.elements) {
    if(this.elements.hasOwnProperty(prop) && (typeof this.elements[prop] != 'function'))
      if(!(this.elements[prop].categories == undefined) && !(this.elements[prop].categories.owner == undefined))
        if(this.elements[prop].categories.owner == ownName)
          this.elements[prop].turnOnEvent();
  }
}

EventsArray.prototype.turnOffEventsOfOwner = function(ownName) {
  for(prop in this.elements) {
    if(this.elements.hasOwnProperty(prop) && (typeof this.elements[prop] != 'function'))
      if(!(this.elements[prop].categories == undefined) && !(this.elements[prop].categories.owner == undefined))
        if(this.elements[prop].categories.owner == ownName)
          this.elements[prop].turnOffEvent();
  }
}

EventsArray.prototype.turnOffAndDeleteEventsOfOwner = function(ownName) {
  for(prop in this.elements) {
    if(this.elements.hasOwnProperty(prop) && (typeof this.elements[prop] != 'function'))
      if(!(this.elements[prop].categories == undefined) && !(this.elements[prop].categories.owner == undefined))
        if(this.elements[prop].categories.owner == ownName) {
          this.elements[prop].markForDeletion();
          delete this.elements[prop];
        }
  }
}
